-- thanks to sweetman for contributing parts of the race store code
races = {}
diff = {}
maxLevel = {}
racescount = 0
penalties = 0
level = 1
currentrace = ""
trackpenalty = false
cache_file = getSetting("Cache Path") .. "race_scores_belgiumFPS.dat"


function writeScore()
    -- save all content deleting the first one
    local f = io.open(cache_file, "w")
    if f == nil then
        log('race score file could not be opened to write scores')
    else
        f:write(level)
        for i,r in ipairs(iraces) do
            f:write("\n"..r[1].."|"..races[r[1]]["besttime"])
        end
        f:close()
        log('race score file written')
    end
end

function readlevel()
    local f = io.open(cache_file, "r")
    if f == nil then
        log("could not read cache_file")
    else
        level = f:read("*n")
        if level == nil then level = 1 end
        f:close()
    end
end


-- return a string:
-- (+3 min 2 sec)
-- (-34 sec)
-- <empty string>
function formatPartialTime(sec)
    local asec = math.abs(sec)
    local m = math.modf (asec / 60)
    local s = math.modf (asec % 60)
    st = string.format("%02.0f",s).." sec"
    if m > 0 then
        st = string.format("%2.0f", m)..":"..st
    end
    if sec > 0.0 then st = " (+"..st..")"
    elseif sec < 0.0 then st = " (-"..st..")"
    else st = ""
    end
    return st
end

function checkPenalty(racename, atchk)
    if not races[racename]['usepenalization'] then return end
    if table.getn(races[racename]['penalties']) >= atchk then
        if races[racename]['penalties'][atchk] then return    end
    end
    races[racename]['penalties'][atchk] = true
    penalties = races[racename]['penaltytime'] + penalties
    trackpenalty = false
end

function formatSeconds(sec)
    local m = math.modf (sec / 60)
    local s = math.modf (sec % 60)
    st = string.format("%02.0f",s).." sec"
    if m > 0 then
        st = string.format("%2.0f", m)..":"..st
    end
    return st
end

function findScore(racename)
    minval = 9999
    local f = io.open(cache_file, "r")
    if f == nil then return minval end
    for line in f:lines() do
        line_ar = explode('|', line)
        if table.getn(line_ar) > 1 then
            if line_ar[1] == racename and tonumber(line_ar[2]) < minval then
                minval = tonumber(line_ar[2])
            end
        end
    end
    f:close()
    log('race score file read')
    return minval
end

function resetRace()
    setupArrow(-1)
    -- reset some values
    if currentrace ~= "" then
        races[currentrace]['checkpointspassed'] = 0
        races[currentrace]['penalties'] = {}
		stopTimer()
    end
    currentrace = ""
    penalties = 0
end

function showInfo(hardness)
	-- This function indicates the user which race to run when crossing a large gate (green/orange/red)
	diff[0] = "easy"
	diff[1] = "medium"
	diff[2] = "hard"
	maxLevel[0] = 3
	maxLevel[1] = 6
	maxLevel[2] = 9
	racesCompleted = level - 1 - maxLevel[hardness] + 3
	if racesCompleted < 0 then racesCompleted = 0 end
	-- if all races in the level have been completed tell the user to try next level
	if level > maxLevel[hardness] then
		if hardness == 3 then
			msg = diff[hardness].." level\nAll races completed, congratulations! Post your times on Rigs on Rods forum!"
		else
			msg = diff[hardness].." level\nAll races completed, congratulations! Try "..diff[hardness + 1].." level."
		end
	-- if no races of this level have been unlocked tell the user which race to complete
	elseif level < maxLevel[hardness] - 2 then
		msg = diff[hardness].." level - locked.\nWin golden medals for all races in "..diff[hardness - 1].." level to unlock."
	-- tell the user how many races he completed  in the present difficulty level
	else
		msg = diff[hardness].." level\nRaces completed: "..racesCompleted.."/3"
	end
    flashMessage(msg, 4)
end

function cheat()
	flashMessage("aw", 4)
end

function loadCheckpoints(race)
    for a,v in ipairs(races[race]['positions']) do
        if a > 1 then
        -- first checkpoint is already spawn
            iname = 'race_'..race..'_'..a
            log("spawn chk number "..a.." of race "..race.." odef file "..v[7].." luaName "..iname)
            spawnObject(v[7], iname, v[1],v[2],v[3], v[4],v[5],v[6], raceEventHandler)
        end
    end
end


    --SetupRace has several parameters:
    --  racename to show on screen (no spaces nor underscore)
    --  array of checkpoints positions
    -- is loop ?
    --  Print Altitude of the next checpoint on screen ?
function setupRace(racename, positions, loop, showaltitude)
    log("setup race "..racename)
    racescount = racescount + 1
    if table.getn(positions) > 1 and racescount <= level then
		spawnObject(positions[1][7], 'race_'..racename..'_1', positions[1][1],positions[1][2],positions[1][3], positions[1][4],positions[1][5],positions[1][6], raceEventHandler)
		log("spawned checkpoint number 1 of race "..racename.." odef file "..positions[1][7].." luaName "..'race_'..racename..'_1' )
    end
    -- add the race finally
    races[racename] = {}
    races[racename]['racename'] = racename
    races[racename]['positions'] = positions
    races[racename]['isloop'] = loop
    races[racename]['startTime'] = 0
    races[racename]['lastcheckpoint'] = 1
    races[racename]['checkpointspassed'] = 1
    races[racename]['numcheckpoints'] = table.getn(positions)
    races[racename]['checkpointsloaded'] = false
    races[racename]['showaltitude'] = showaltitude
    races[racename]['usepenalization'] = false
    races[racename]['penaltytime'] = 0.0
    races[racename]['penalties'] = {}
    races[racename]["goldentime"] = 0.0
    races[racename]["platetime"] = 0.0
    races[racename]["bronzetime"] = 0.0
    races[racename]["idxrace"] = racescount
    races[racename]['besttime'] = findScore(racename)
    if loop then
        races[racename]['goalnum'] = 1
    else
        races[racename]['goalnum'] = table.getn(positions)
    end

end

function nextRace(something)
    for a,v in ipairs(iraces) do
        if races[v[1]]["idxrace"] == something then
            local iname = 'race_'..v[1]..'_1'
            local position = races[v[1]]["positions"][1]
            spawnObject(position[7], iname, position[1],position[2],position[3], position[4],position[5],position[6], raceEventHandler)
            log("spawn chk number 1 of race "..v[1].." odef file "..position[7].." luaName "..iname)
			return
        end
    end
end


function getMedal(racename, spent)
    if spent <= races[racename]["goldentime"] then
        return "Golden"
    elseif spent <= races[racename]["platetime"] then
        return "Plate"
    elseif spent <= races[racename]["bronzetime"] then
        return "Bronze"
    else
        return "\t"
    end
end

function getCheckPointFromName(instance)
    ar = explode('_', instance)
    if table.getn(ar) < 3 then return -1
    else
        return tonumber(ar[3])
    end
end

function getRacenameFromName(instance)
    ar = explode('_', instance)
    if table.getn(ar) < 2 then return ""
    else
        return tostring(ar[2])
    end
end

function setupArrow(position)
    if position == -1 or position == nil then
        -- hide it
        pointArrow(0, 0, 0, "")
    else
        txt = currentrace.."\ncheckpoint "..tostring(position).." / "..races[currentrace]['numcheckpoints']
        if position == 1 and races[currentrace]['isloop'] then
            txt = "Finish"
        end
        data = races[currentrace]['positions'][position]
        if data then
            if races[currentrace]["showaltitude"] then
                txt = txt .. "\n\nAlt "..string.format("%.0f", races[currentrace]['positions'][position][2] * 3,2808399 ).." feets"
            end
            pointArrow(data[1], data[2], data[3], txt)
        else
            pointArrow(0, 0, 0, "")
        end
    end
end

function raceEventHandler(instance, box)
    checknum = getCheckPointFromName(instance)
    racename = getRacenameFromName(instance)

    if box == "resetcurrentrace"  then
        resetRace()
    elseif box == "penalty" then
        trackpenalty = true
		flashMessage("PENALTY!",2)
	elseif box == "showinfogreen" then
		showInfo(0)
	elseif box == "showinfoorange" then
		showInfo(1)
	elseif box == "showinfored" then
		showInfo(2)
	elseif box == "cheat" then
		showInfo(0)
    else
        -- create an array with all required names in it
        if currentrace == "" then
            -- not racing currently
            if checknum == 1 then
                -- start of the race
                currentrace = racename
                races[currentrace]['startTime'] = getTime()
                races[currentrace]['lastcheckpoint'] = 1
                races[currentrace]['checkpointspassed'] = 1
                if not races[racename]['checkpointsloaded'] then
                    loadCheckpoints(racename)
                    races[racename]['checkpointsloaded'] = true
                end
                if trackpenalty then checkPenalty(racename, checknum) end
                setupArrow(2)
                startTimer()
                msg = "Welcome to Belgium Championship\nRace "..races[currentrace]["idxrace"].." of ".. racescount.. " started!"
                flashMessage(msg, 4)
            end
        elseif currentrace == racename then
            -- we hit a checkpoint from the same race!
            if checknum == races[currentrace]['lastcheckpoint'] + 1 then
                -- racing, correct checkpoint!
                if checknum == races[currentrace]['numcheckpoints'] and races[currentrace]['isloop'] then
                    setupArrow(1)
                else
                    setupArrow(checknum+1)
                end
                races[currentrace]['lastcheckpoint'] = checknum
                races[currentrace]['checkpointspassed'] = races[currentrace]['checkpointspassed'] + 1
                if trackpenalty then checkPenalty(racename, checknum) end
                time = getTime() - races[currentrace]['startTime']
                msg = "Completed: "..checknum.."/"..(races[currentrace]['numcheckpoints']).." in "..formatSeconds(time)
                flashMessage(msg, 4)
            else
                if not checknum == races[currentrace]['lastcheckpoint'] then
                    -- racing, wrong checkpoint, prevent trigger of the same checkpoint twice
                    flashMessage("Wrong gate! You must find and pass Checkpoint "..races[currentrace]['lastcheckpoint'] + 1, 4)
                end
            end
            if checknum == races[currentrace]['goalnum'] and races[currentrace]['checkpointspassed'] == races[racename]['numcheckpoints'] then
                -- racing, reached finish
                --time = getTime() - starttime
                line1 = ""
                line2 = ""
                time = stopTimer()
                if races[racename]['usepenalization']  then
                    line1 = "penalty \t\t\t "..line1 --one more tab than line2
                    line2 = formatSeconds(penalties).."\t\t "
                end
                line1 = "Medal \t\ttime\t\t".. line1.."final time"
                line2 = getMedal(racename, time).. "\t\t"..formatSeconds(time).."\t\t"..line2..formatSeconds(time + penalties)
                time = time + penalties
                local savescores = false
                if time < races[currentrace]["goldentime"] and level < 9 then
                    level = level + 1
                    nextRace(level)
                    savescores = true
                end
                if time < tonumber(races[currentrace]['besttime']) then
                    line2 = line2 .. "\nNew Best Lap Time!"
                    if tonumber(races[currentrace]['besttime']) ~= 9999 then
                        line2 = line2.. formatPartialTime(time - races[currentrace]['besttime'])
                    end
                    races[currentrace]['besttime'] = time
                    savescores = true
                    if time < races[currentrace]["goldentime"] and level < 9 then
                        line2 = line2.."\n Unlocked next race, good work!"
                    end
                else
                    line2 = line2 .. "\n (Best Lap: "..formatSeconds(races[currentrace]['besttime'])..")"
                end
                if savescores then writeScore() end
                log(currentrace.." finished. Score:\n"..line1.."\n"..line2)
                flashMessage(line1.."\n"..line2, 15)
                resetRace()
            end
        else
            -- flashMessage("This Checkpoint belongs to race"..racename.."!", 4)
        end
    end
end

------------------------------------------------------------------------------------------------------------
--array of checkpoints  to use: x, y, z, rotX, rotY, rotZ
race_private_property = {
{681.069519, 5.091340, 622.072266, 0.000000, 94.500000, 0.000000, "baja_gate1"}, 
{659.546143, 2.745841, 618.136719, 0.000000, 25.500000, 0.000000, "baja_gate1"}, 
{874.164978, 2.713794, 400.741516, 0.000000, -35.500000, 0.000000, "baja_gate1"}, 
{890.369263, 3.400290, 299.995758, 0.000000, 25.000000, 0.000000, "baja_gate1"}, 
{678.132324, 8.370727, 93.202545, 0.000000, 46.500000, 0.000000, "baja_gate1"}, 
{429.766785, 8.300570, 146.034149, 0.000000, 120.500000, 0.000000, "baja_gate1"}, 
{341.707397, 8.392775, 190.811111, 0.000000, 44.000000, 0.000000, "baja_gate1"}, 
{272.489105, 16.579178, 140.031219, 0.000000, 93.000000, 0.000000, "baja_gate1"}, 
{166.150223, 17.607267, 117.767967, 0.000000, 58.000000, 0.000000, "baja_gate1"}, 
{122.775131, 35.381340, 80.397972, 0.000000, 131.000000, 0.000000, "baja_gate1"}, 
{91.471535, 7.313995, 136.956421, 0.000000, 193.000000, 0.000000, "baja_gate1"}, 
{60.386009, 3.598121, 274.143677, 0.000000, 193.000000, 0.000000, "baja_gate1"}, 
{138.095078, 41.170563, 281.666779, 0.000000, 140.000000, 0.000000, "baja_gate1"}, 
{183.280243, 12.118886, 205.927124, 0.000000, 140.000000, 0.000000, "baja_gate1"}, 
{225.621033, 12.648453, 196.042206, 0.000000, 197.000000, 0.000000, "baja_gate1"}, 
{194.108688, 11.853185, 257.186371, 0.000000, 197.000000, 0.000000, "baja_gate1"}, 
{211.511932, 33.294762, 314.247772, 0.000000, 197.000000, 0.000000, "baja_gate1"}, 
{174.632050, 38.762589, 311.930359, 0.000000, 256.000000, 0.000000, "baja_gate1"}, 
{100.554543, 10.662681, 315.337341, 0.000000, 259.000000, 0.000000, "baja_gate1"}
}

race_mud_trial={
{687.028625, 4.274662, 636.731384, 0.000000, 130.500000, 0.000000, "baja_gate1"}, 
{659.695984, 2.895114, 619.408203, 0.000000, 24.500000, 0.000000, "baja_gate1"}, 
{559.159790, 3.014727, 634.706238, 0.000000, 97.500000, 0.000000, "baja_gate1"}, 
{502.199402, 6.576231, 712.437012, 0.000000, -179.500000, 0.000000, "baja_gate1"}, 
{507.830658, 7.179049, 977.136902, 0.000000, -171.000000, 0.000000, "baja_gate1"}, 
{548.344, 10.9377, 984.265, 0, 360.875, 0, "baja_gate1"}, 
{565.289, 19.9612, 934.447, 0, 301.907, 0, "dgchk"}, 
{588.188, 18.3657, 920.763, 0, 333.879, 0, "dgchk"}, 
{584.755, 17.4427, 905.373, 0, 409.853, 0, "dgchk"}, 
{587.267, 17.8128, 896.554, 0, 282.771, 0, "dgchk"}, 
{604.085, 12.5783, 903.662, 0, 227.538, 0, "dgchk"}, 
{612.196, 17.8498, 917.562, 0, 174.0233, 0, "dgchk"}, 
{603.283, 23.2425, 930.79, 0, 179.4089, 0, "dgchk"}, 
{603.396, 13.1924, 959.672, 0, 179.4769, 0, "dgchk"}, 
{620.303, 10.9377, 983.658, 0, 271.952, 0, "dgchk"}, 
{640.398, 11.2746, 958.344, 0, 353.656, 0, "dgchk"}, 
{628.885, 11.5194, 917.117, 0, 395.825, 0, "dgchk"}, 
{624.934, 10.6642, 905.14, 0, 336.238, 0, "dgchk"}, 
{627.224, 6.76025, 889.049, 0, 419.431, 0, "dgchk"}, 
{610.244, 9.85842, 878.958, 0, 398.805, 0, "dgchk"}, 
{602.971, 7.10948, 861.045, 0, 410.15, 0, "dgchk"}, 
{585.998, 8.84587, 860.693, 0, 128.7594, 0, "dgchk"}, 
{565.771, 15.1742, 870.599, 0, 102.0597, 0, "dgchk"}, 
{548.491, 18.6885, 883.87, 0, 151.1049, 0, "dgchk"}, 
{560.047, 20.3682, 897.744, 0, 201.869, 0, "dgchk"}, 
{554.172, 16.9534, 920.482, 0, 125.6653, 0, "dgchk"}, 
{533.925, 16.3353, 924.899, 0, 93.46504, 0, "dgchk"}, 
{522.888, 15.7056, 932.238, 0, 139.6455, 0, "dgchk"}, 
{515.72, 10.9377, 957.782, 0, 170.1269, 0, "baja_gate1"}
}

race_field_trial={
{701.682312, 4.545968, 649.822205, 0.000000, 139.500000, 0, "baja_gate1"}, 
{676.31, 4.15463, 730.481, 0, 155.875, 0, "baja_gate1"}, 
{664.882, 6.27247, 746.739, 0, 182.231, 0, "dgchk"}, 
{680.564, 7.79706, 750.851, 0, 271.154, 0, "dgchk"}, 
{698.886, 15.7528, 748.243, 0, 273.904, 0, "dgchk"}, 
{716.158, 10.0084, 747.953, 0, 260.726, 0, "dgchk"}, 
{731.256, 6.61965, 760.334, 0, 197.701, 0, "dgchk"}, 
{721.92, 4.8811, 767.749, 0, 91.13066, 0, "dgchk"}, 
{693.209, 4.54801, 768.315, 0, 91.13066, 0, "dgchk"}, 
{685.544, 4.82969, 774.556, 0, 187.3198, 0, "dgchk"}, 
{691.426, 4.28239, 785.45, 0, 243.814, 0, "dgchk"}, 
{702.56, 8.53181, 787.646, 0, 272.576, 0, "dgchk"}, 
{724.383, 2.15212, 793.001, 0, 178.7258, 0, "dgchk"}, 
{729.569, 4.90354, 804.038, 0, 233.501, 0, "dgchk"}, 
{743.014, 5.9057, 812.06, 0, 227.084, 0, "dgchk"}, 
{733.815, 6.1031, 824.531, 0, 427.572, 0, "dgchk"}, 
{720.09, 7.65637, 819.583, 0, 433.073, 0, "dgchk"}, 
{707.879, 11.722, 817.772, 0, 433.645, 0, "dgchk"}, 
{691.771, 4.85937, 807.774, 0, 399.726, 0, "dgchk"}, 
{674.576, 4.82368, 785.951, 0, 432.958, 0, "dgchk"}, 
{661.027, 4.26932, 780.654, 0, 424.249, 0, "dgchk"}, 
{642.032, 7.81231, 762.148, 0, 107.7944, 0, "dgchk"}, 
{630.595, 5.39744, 773.047, 0, 122.5767, 0, "dgchk"}, 
{620.144, 3.13483, 777.766, 0, 99.31453, 0, "dgchk"}, 
{609.365, 9.43634, 782.454, 0, 148.7035, 0, "dgchk"}, 
{616.064, 6.16444, 791.484, 0, 244.044, 0, "dgchk"}, 
{613.685, 6.5626, 804.68, 0, 239.002, 0, "dgchk"}, 
{632.613, 2.47343, 799.018, 0, 358.865, 0, "dgchk"}, 
{633.23, 8.10659, 790.758, 0, 335.947, 0, "dgchk"}, 
{642.961, 9.70523, 787.705, 0, 263.066, 0, "dgchk"}, 
{653.618, 5.5425, 793.758, 0, 197.405, 0, "dgchk"}, 
{652.232, 5.5495, 807.386, 0, 183.0816, 0, "dgchk"}, 
{649.929, 8.13345, 814.089, 0, 108.5971, 0, "dgchk"}, 
{640.09, 2.88417, 834.507, 0, 183.6549, 0, "dgchk"}, 
{649.873, 3.05849, 845.522, 0, 260.202, 0, "dgchk"}, 
{662.362, 11.4846, 860.974, 0, 181.4779, 0, "dgchk"}, 
{675.832, 6.29924, 869.177, 0, 266.963, 0, "dgchk"}, 
{679.798, 6.08982, 861.949, 0, 399.087, 0, "dgchk"}, 
{675.288, 3.24852, 843.562, 0, 388.086, 0, "dgchk"}, 
{659.927, 7.43259, 832.083, 0, 416.62, 0, "dgchk"}, 
{664.548, 5.05714, 822.748, 0, 308.675, 0, "dgchk"}, 
{664.743, 4.35662, 810.481, 0, 370.096, 0, "dgchk"}, 
{678.585, 1.91409, 812.72, 0, 227.658, 0, "dgchk"}, 
{688.706, 4.89585, 821.943, 0, 227.658, 0, "dgchk"}, 
{699.083, 8.02569, 831.397, 0, 227.658, 0, "dgchk"}, 
{696.721, 8.6292, 840.446, 0, 170.2482, 0, "dgchk"}, 
{697.218, 3.03137, 853.6, 0, 208.636, 0, "dgchk"}, 
{709.879, 5.47361, 857.323, 0, 272.349, 0, "dgchk"}, 
{726.078, 12.5783, 850.561, 0, 290.455, 0, "dgchk"}, 
{749.55, 1.3838, 835.352, 0, 310.05, 0, "dgchk"}, 
{759.752, 6.52615, 822.396, 0, 347.407, 0, "dgchk"}, 
{761.676, 6.02161, 795.378, 0, 401.609, 0, "baja_gate1"}
}

race_hills_trail={
{720.613525, 3.687873, 664.471191, 0.000000, 148.000000, 0.000000, "baja_gate1"}, 
{659.856995, 2.558736, 620.250671, 0.000000, 19.500000, 0.000000, "baja_gate1"}, 
{739.515625, 5.631189, 550.071289, 0.000000, -32.000000, 0.000000, "baja_gate1"}, 
{886.719727, 2.460975, 386.077118, 0.000000, -22.000000, 0.000000, "baja_gate1"}, 
{781.840454, 6.670712, 206.491882, 0.000000, 50.500000, 0.000000, "baja_gate1"}, 
{623.587036, 8.203250, 98.167542, 0.000000, 109.000000, 0.000000, "baja_gate1"}, 
{528.120117, 8.476692, 162.757248, 0.000000, 0.000000, 0.000000, "dgchk"}, 
{535.754333, 8.419278, 243.046356, 0.000000, 0.000000, 0.000000, "dgchk"}, 
{529.108398, 8.633583, 334.501587, 0.000000, 47.000000, 0.000000, "dgchk"}, 
{550.303650, 12.366072, 359.457245, 0.000000, -31.000000, 0.000000, "baja_gate1"}, 
{508.207367, 38.824623, 431.122589, 0.000000, -82.000000, 11.000000, "baja_gate1"}, 
{416.303925, 13.447850, 446.336639, 0.000000, -75.000000, 0.000000, "baja_gate1"}, 
{440.037720, 32.962624, 485.585938, 0.000000, -136.000000, 0.000000, "baja_gate1"}, 
{476.826508, 27.677942, 550.657898, 0.000000, -136.000000, 0.000000, "baja_gate1"}, 
{545.689575, 45.301956, 470.548828, 0.000000, -186.000000, 0.000000, "baja_gate1"}, 
{604.011108, 47.183708, 492.254791, 0.000000, -138.000000, 0.000000, "baja_gate1"}, 
{651.704163, 30.759916, 516.080139, 0.000000, -60.000000, 0.000000, "baja_gate1"}, 
{683.740601, 31.716734, 458.861206, 0.000000, -123.000000, 0.000000, "baja_gate1"}, 
{650.695496, 12.946462, 415.216858, 0.000000, -123.000000, 0.000000, "baja_gate1"}, 
{605.767395, 37.271278, 420.112122, 0.000000, -108.000000, 0.000000, "baja_gate1"}, 
{615.319824, 33.472099, 372.400635, 0.000000, -194.000000, 0.000000, "baja_gate1"}, 
{603.155823, 11.329178, 330.782593, 0.000000, -248.000000, 0.000000, "baja_gate1"}
}

race_fields_trail={
{736.646423, 4.263981, 669.639832, 0.000000, -175.500000, 0.000000, "baja_gate1"}, 
{659.856995, 2.558736, 620.250671, 0.000000, 19.500000, 0.000000, "baja_gate1"}, 
{628.282593, 2.930354, 641.255859, 0, 211, 0.000000, "baja_gate1"}, 
{691.634766, 1.428856, 701.666565, 0, 211, 0.000000, "baja_gate1"}, 
{747.753784, 1.498302, 723.516541, 0, 318, 0.000000, "baja_gate1"}, 
{863.693481, 1.195159, 609.231873, 0, 318, 0.000000, "baja_gate1"}, 
{770.746216, 5.360556, 526.513611, 0, 58, 0.000000, "baja_gate1"}
}



race_wood_walk={
{755.353088, 4.265049, 663.742249, 0.000000, -158.000000, 0.000000, "baja_gate1"}, 
{659.780029, 2.873029, 620.282166, 0.000000, 30.000000, 0.000000, "baja_gate1"}, 
{750.624329, 5.755165, 540.756958, 0.000000, -41.500000, 0.000000, "baja_gate1"}, 
{892.996826, 2.529007, 373.776550, 0.000000, 0.000000, 0.000000, "baja_gate1"}, 
{837.340271, 3.808097, 249.037079, 0.000000, 52.500000, 0.000000, "baja_gate1"}, 
{804.279724, 7.139695, 241.372543, 0, 145, 0.000000, "baja_gate1"}, 
{827.851929, 6.554274, 272.932892, 0, -115, 0.000000, "dgchk"}, 
{858.233521, 8.234262, 324.64859, 0, -137, 0.000000, "dgchk"}, 
{843.648438, 7.357698, 361.315247, 0, -106, 0.000000, "dgchk"}, 
{805.821167, 12.330078, 328.303894, 0, -106, 0.000000, "dgchk"}, 
{783.983276, 3.989402, 324.719849, 0, -140, 0.000000, "dgchk"}, 
{814.336487, 11.906389, 381.390625, 0, -238, 0.000000, "dgchk"}, 
{778.053345, 7.716596, 359.959473, 0, -261, 0.000000, "dgchk"}, 
{749.795288, 18.605289, 372.903198, 0, -277, 0.000000, "dgchk"}, 
{760.43, 17.9088, 313.578, 0, 185.043, 0, "dgchk"}, 
{738.52, 27.7821, 305.112, 0, -62.5422, 0, "dgchk"}, 
{708.302, 30.246, 330.325, 0, -82.59563, 0, "dgchk"}, 
{683.219, 31.314, 328.237, 0, -27.9357, 0, "dgchk"}, 
{662.806, 12.7272, 363.122, 0, 240.62, 0, "dgchk"}, 
{657.55, 31.8733, 280.525, 0, 155.708, 0, "dgchk"}, 
{693.839, 32.3032, 279.94, 0, 93.255, 0, "dgchk"}, 
{748.803, 26.8863, 273.924, 0, 64.378, 0, "dgchk"}, 
{771.575, 17.7236, 270.624, 0, 178.855, 0, "dgchk"}, 
{746.714, 24.3763, 215.677, 0, 229.848, 0, "dgchk"}, 
{712.723, 33.0864, 212.218, 0, -89.357086, 0, "dgchk"}, 
{646.94, 44.167, 237.766, 0, -9.7157, 0, "dgchk"}, 
{642.052, 30.6425, 266.318, 0, -9.7157, 0, "dgchk"}, 
{612.946, 33.3246, 273.113, 0, 241.881, 0, "dgchk"}, 
{601.817, 26.1974, 251.161, 0, 187.794, 0, "dgchk"}, 
{619.017, 38.4916, 224.713, 0, 165.907, 0, "dgchk"}, 
{630.018, 40.0237, 169.547, 0, 169.917, 0, "dgchk"}, 
{664.706, 35.2306, 157.405, 0, 144.134, 0, "dgchk"}, 
{670.443, 13.1252, 117.415, 0, 222.744, 0, "dgchk"}, 
{621.292, 28.9848, 148.835, 0, -29.7691, 0, "dgchk"}, 
{571.51, 14.0775, 142.073, 0, -77.668, 0, "dgchk"}, 
{544.685, 10.7934, 165.737, 0, 28.788, 0, "dgchk"}, 
{590.389, 25.4642, 201.724, 0, 2.3171, 0, "dgchk"}, 
{544.324, 9.77371, 262.726, 0, -68.042, 0, "baja_gate1"}
}

race_hard_me={
{785.177612, 4.820050, 646.599976, 0.000000, -163.000000, 0.000000, "baja_gate1"}, 
{659.895691, 2.677112, 620.155151, 0.000000, 18.500000, 0.000000, "baja_gate1"}, 
{558.672791, 2.932861, 635.232361, 0.000000, 90.500000, 0.000000, "baja_gate1"}, 
{505.574341, 4.017795, 658.371765, 0.000000, 129.000000, 0.000000, "baja_gate1"}, 
{533.807, 7.38293, 699.18, 0, -51.8177, 0, "baja_gate1"}, 
{488.348, 4.92195, 676.557, 0, -71.9857, 0, "baja_gate1"}, 
{435.31, 26.0859, 692.152, 0, -33.9414, 0, "dgchk"}, 
{418.993, 43.5201, 733.493, 0, -31.3058, 0, "dgchk"}, 
{386.388, 20.5196, 743.007, 0, -84.13249, 0, "dgchk"}, 
{339.885, 11.4846, 753.535, 0, -81.03856, 0, "dgchk"}, 
{309.829, 20.7092, 784.576, 0, 261.429, 0, "dgchk"}, 
{222.623, 34.1988, 773.661, 0, 266.127, 0, "dgchk"}, 
{194.775, 42.3835, 769.529, 0, 269.83, 0, "dgchk"}, 
{143.541, 32.5357, 766.636, 0, 243.359, 0, "dgchk"}, 
{123.104, 14.061, 754.204, 0, 247.256, 0, "dgchk"}, 
{138.666, 12.0314, 719.86, 0, 134.45, 0, "dgchk"}, 
{145.997, 11.7648, 686.01, 0, 140.753, 0, "dgchk"}, 
{168.718, 12.9566, 669.066, 0, 103.808, 0, "dgchk"}, 
{194.939, 9.23512, 671.827, 0, 116.9523, 0, "dgchk"}, 
{141.155, 2.34993, 703.438, 0, -63.1625, 0, "dgchk"}, 
{119.326, 1.59497, 715.983, 0, -56.5161, 0, "dgchk"}, 
{93.0922, 2.93334, 756.125, 0, -1.6267, 0, "dgchk"}, 
{121.301, 3.90247, 810.087, 0, 42.491, 0, "dgchk"}, 
{145.063, 14.4324, 824.449, 0, 58.763, 0, "dgchk"}, 
{192.329, 2.0518, 856.72, 0, 47.304, 0, "baja_gate1"}
}


race_forest_trail={
{799.339844, 5.087048, 638.583740, 0.000000, -139.000000, 0.000000, "baja_gate1"}, 
{659.895691, 2.677112, 620.155151, 0.000000, 18.500000, 0.000000, "baja_gate1"}, 
{558.672791, 2.932861, 635.232361, 0.000000, 90.500000, 0.000000, "baja_gate1"}, 
{505.574341, 4.017795, 658.371765, 0.000000, 129.000000, 0.000000, "baja_gate1"}, 
{445.359741, 5.657617, 615.316528, 0.000000, 48.500000, 0.000000, "baja_gate1"}, 
{294.366882, 15.622364, 492.846100, 0.000000, 44.000000, -1.000000, "baja_gate1"}, 
{218.950607, 8.328753, 443.648682, 0.000000, 135.500000, 0.000000, "baja_gate1"}, 
{173.283615, 10.264071, 520.028748, 0, 246, 0.000000, "baja_gate1"}, 
{231.309692, 26.632252, 549.535706, 0, 231, 0.000000, "dgchk"}, 
{294.076385, 43.755192, 587.558594, 0, 268, 0.000000, "dgchk"}, 
{315.286011, 10.743589, 659.111389, 0, 184, 0.000000, "dgchk"}, 
{213.075821, 13.61153, 671.39325, 0, 109, 0.000000, "dgchk"}, 
{230.392548, 23.883999, 711.204773, 0, -95, 0.000000, "dgchk"}, 
{339.826874, 6.177271, 736.308533, 0, -95, 0.000000, "dgchk"}, 
{337.336823, 11.05376, 828.11969, 0, 85, 0.000000, "dgchk"}, 
{267.085144, 1.999588, 814.250427, 0, -25, 0.000000, "dgchk"}, 
{195.214096, 1.94767, 871.847473, 0, 176, 0.00000, "dgchk"}, 
{250.438477, 1.041576, 904.510864, 0, 282, 0.000000, "dgchk"}, 
{339.516052, 27.091248, 875.925415, 0, 251, 0.000000, "dgchk"}, 
{472.332794, 10.483687, 970.062439, 0, 96, 0.000000, "baja_gate1"}
}



race_rock_crawling={
{808.830566, 5.019064, 624.658508, 0.000000, -109.500000, 0.000000, "baja_gate1"}, 
{659.780029, 2.873029, 620.282166, 0.000000, 30.000000, 0.000000, "baja_gate1"}, 
{750.624329, 5.755165, 540.756958, 0.000000, -41.500000, 0.000000, "baja_gate1"}, 
{892.996826, 2.529007, 373.776550, 0.000000, 0.000000, 0.000000, "baja_gate1"}, 
{837.340271, 3.808097, 249.037079, 0.000000, 52.500000, 0.000000, "baja_gate1"}, 
{704.676819, 8.202519, 106.615601, 0.000000, 74.000000, 0.000000, "baja_gate1"}, 
{772.603149, 5.714613, 148.393372, 0.000000, 62.000000, 0.000000, "baja_gate1"}, 
{827.621094, 3.967069, 183.521881, 0.000000, 43.000000, 0.000000, "baja_gate1"}, 
{893.752075, 3.826091, 231.409592, 0.000000, 43.000000, 0.000000, "baja_gate1"}, 
{959.761597, 8.710119, 289.200256, 0.000000, 43.000000, 0.000000, "baja_gate1"}, 
{975.644775, 9.149279, 329.612396, 0.000000, 131.000000, 0.000000, "baja_gate1"}, 
{946.038513, 5.328468, 342.810730, 0.000000, 131.000000, 0.000000, "baja_gate1"}
}


function init()
    readlevel()
    -- spawn Stage pole that inform user what race is the next one.
    spawnObject("dgstartgreen", "dgstartgreen", 728.514771, 5.741645, 610.253540, 0.000000, 122.000000, -1.000000, raceEventHandler)
    spawnObject("dgstartorange", "dgstartorange", 749.136047, 6.025445, 626.739990, 0.000000, 167.000000, -1.000000, raceEventHandler)
    spawnObject("dgstartred", "dgstartred", 777.517944, 6.287702, 614.447998, 0.000000, 215.000000, -1.000000, raceEventHandler)
    --spawnObject(oname, iname, v[1],v[2],v[3], v[4],v[5],v[6], raceEventHandler)
    -- 1
    local race = "PrivateProperty"
    setupRace(race, race_private_property, false, false)
    races[race]["goldentime"] = 286.07
    races[race]["platetime"]  = races[race]["goldentime"] + 60
    races[race]["bronzetime"] = races[race]["platetime"] + 60
    log("LUA Private property loaded")
    -- 2
    race = "MudTrial"
    setupRace(race, race_mud_trial, false, false)
    races[race]["usepenalization"] = true
    races[race]["penaltytime"] = 60.0
    races[race]["goldentime"] = 240.01
    races[race]["platetime"]  = races[race]["goldentime"] + 60
    races[race]["bronzetime"] = races[race]["platetime"] + 60

    log("LUA Mud Trial loaded")
    --3
    race = "FieldTrial"
    setupRace(race, race_field_trial, false, false)
    races[race]["usepenalization"] = true
    races[race]["penaltytime"] = 4.0
    races[race]["goldentime"] = 329.01
    races[race]["platetime"]  = races[race]["goldentime"] + 60
    races[race]["bronzetime"] = races[race]["platetime"] + 60

    log("LUA Fields Trial loaded")

    -- 4
    race = "HillsTrail"
    setupRace(race, race_hills_trail, false, false)
    races[race]["usepenalization"] = true
    races[race]["penaltytime"] = 5.0
    races[race]["goldentime"] = 460.75
    races[race]["platetime"]  = races[race]["goldentime"] + 45
    races[race]["bronzetime"] = races[race]["platetime"] + 45
    log("LUE Hills Trails loaded")

    -- 5
    race = "FieldsTrail"
    setupRace(race, race_fields_trail, false, false)
    races[race]["usepenalization"] = true
    races[race]["penaltytime"] = 5.0
    races[race]["bronzetime"] = 216.39
    races[race]["platetime"]  = races[race]["goldentime"] + 60
    races[race]["bronzetime"] = races[race]["platetime"] + 60
    log("LUA Fields Trails loaded")


    -- 6
    race = "WoodWalk"
    setupRace(race, race_wood_walk, false, false)
    races[race]["usepenalization"] = true
    races[race]["goldentime"] = 655.28
    races[race]["platetime"]  = races[race]["goldentime"] + 30
    races[race]["bronzetime"] = races[race]["platetime"] + 30
    races[race]["penaltytime"] = 5.0
    log("LUA Wood Walk loaded")

    -- 7
    race = "HardMe"
    setupRace(race, race_hard_me, false, false)
    races[race]["usepenalization"] = true
    races[race]["penaltytime"] = 7.0
    races[race]["goldentime"] = 320.33
    races[race]["platetime"]  = races[race]["goldentime"] + 20
    races[race]["bronzetime"] = races[race]["platetime"] + 20

    log("LUA Hard Me loaded")
    -- 8
    race = "WoodWalk2"
    setupRace(race, race_forest_trail, false, false)
    races[race]["usepenalization"] = true
    races[race]["penaltytime"] = 7.0
    races[race]["goldentime"] = 385.01
    races[race]["platetime"]  = races[race]["goldentime"] + 20
    races[race]["bronzetime"] = races[race]["platetime"] + 20

    log("LUA Forest Trails loaded")
    --9
    race="RockCrawling"
    setupRace(race, race_rock_crawling, false, false )
    races[race]["usepenalization"] = true
    races[race]["penaltytime"] = 10.0
    races[race]["goldentime"] = 222.70
    races[race]["platetime"]  = races[race]["goldentime"] + 5
    races[race]["bronzetime"] = races[race]["platetime"] + 10

    log("LUA Rock Crawling loaded")
    iraces = {
            {"PrivateProperty"},
            {"MudTrial"},
			{"FieldTrial"},
            {"HillsTrail"},
            {"FieldsTrail"},
            {"WoodWalk"},
            {"HardMe"},
            {"WoodWalk2"},
            {"RockCrawling"}
    }

end

init()
log("lua file loaded")
